import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import java.io.*;
import java.net.*;

/**
 * This is the main control panel for the
 * spirograph renderer.  It has a sort
 * of vertical layout, and is intended to be
 * placed along the east or west side of the
 * primary DesignPanel.
 *
 * The following pieces make up the control panel:
 *
 *    - (Design center point label)
 *    - Center x coordinate (textfield, with label)
 *    - Center y coordinate (textfield, with label)
 *    - (center chooser pick label)
 *    - (spacer)
 *    - (Stabile parameters label)
 *    - Stabile type picker (choice, with label)
 *    - Stabile size  (textfield, with label)
 *    - (spacer)
 *    - (Mobile parameters label)
 *    - Mobile size (textfield, with label)
 *    - Pen radius (textfield, with label)
 *    - Revolutions (textfield, with label)
 *    - Outside/inside (two radio buttons)
 *    - Pen size and softness (2 textfields, 2 labels)
 *    - Pen color (label and button)
 *    - (spacer) 
 *    - (Advanced parameters label)
 *    - Dots per revolution (textfield, with label)
 *    - Adaptive (checkbox)
 *    - Paper color (label and button)
 *    - (spacer)
 *    - Presets (choice, with label)
 *    - (spacer)
 *    - Render button (Button, bigger font!)
 *    - Help button (Button, normal font)
 *    - Close button (Button, normal font)
 *
 * To support the notion of presets, 
 */

public class ControlPanel extends Panel
  implements ActionListener,TextListener,ItemListener,WindowListener,
	     MouseMotionListener,MouseListener,Observer
{
    public static final double BASE_CSW = 10.0;
    
    public static final String CENTERX = "centerx";
    public static final String CENTERY = "centery";
    public static final String STABILETYPE = "stabiletype";
    public static final String STABILESIZE = "stabilesize";
    public static final String MOBILESIZE = "mobilesize";
    public static final String PENSIZE = "pensize";
    public static final String REVOLUTIONS = "revolutions";
    public static final String SIDEMODE = "sidemode";
    public static final String PENSOLIDDIAMETER = "pensoliddiameter";
    public static final String PENFADEDIAMETER = "penfadediameter";
    public static final String PENCOLOR = "pencolor";
    public static final String PENCOLORPICK = "pencolorpick";
    public static final String DOTSPER = "dotsper";
    public static final String ADAPTIVE = "adaptive";
    // public static final String PAPERCOLOR = "papercolor";
    // public static final String PAPERCOLORPICK = "papercolorpick";

    public static final String RENDER = "render";
    public static final String HELP = "help";
    public static final String CLOSE = "close";
    

    class ControlItem {
	String name;
	Component comp;
	boolean checkNumericValue;
	boolean needsScale;
	double min;
	double max;
	ControlItem(String n, Component c,
		    boolean cnv, double mn, double mx, boolean ns)
	{
	    name = n;
	    comp = c;
	    checkNumericValue = cnv;
	    min = mn; max = mx;
	    needsScale = ns;
	    components.put(c, this);
	    componentsByName.put(n, this);
	}

	Color getBackground() { return comp.getBackground(); }
	double getDoubleValue() {
	    if (comp instanceof TextField) {
		double v = 0.0;
		try {
		    v = Double.valueOf(((TextField)comp).getText()).doubleValue();
		}
		catch (NumberFormatException e) { }
		return v;
	    }
	    else return 0.0;
	}

	boolean validate() {
	    boolean ret = true;
	    if (checkNumericValue) {
		double v = 0.0;
		if (comp instanceof TextField) {
		    try {
			v = Double.valueOf(((TextField)comp).getText()).doubleValue();
		    }
		    catch (NumberFormatException e) {
			ret = false;
		    }
		    if (v < min || v > max) ret = false;
		    if (ret) {
			((TextField)comp).setForeground(Color.black);
		    }
		    else {
			((TextField)comp).setForeground(Color.red);
		    }
		}
	    }
	    return ret;
	}
    }


    Panel innerPanel;
    Hashtable components;
    Hashtable componentsByName;
    Choice presets;
    DesignPanel myDesignPanel;
    ColorPicker  penColorPicker;
    double canvasSizeWorld;
    Frame parentFrame;
    Applet parentApplet;
    Font smallFont;
    Font smallBold;
    Font mediumFont;
    Dimension canvasSize;
    Color  penColor;
    boolean listForPresets;

    ControlItem getItemByName(String n) {
	return (ControlItem)(componentsByName.get(n));
    }
    ControlItem getItem(Component c) {
	if (c == null) return null;
	else return (ControlItem)(components.get(c));
    }
    boolean validateItem(Component c) {
	if (c == null) return true;
	ControlItem ci = getItem(c);
	if (ci != null)
	    return ci.validate();
	else
	    return true;
    }

    public ControlPanel(DesignPanel dp, double csw, 
			Frame parent, Applet applet, boolean listEm) 
    {
	int smFontSize = 12;
	int lgFontSize = 14;
	parentFrame = parent;
	parentApplet = applet;
	canvasSizeWorld = csw;
	setLayout(new BorderLayout());
	setBackground(Color.lightGray);
	innerPanel = new Panel();
	components = new Hashtable();
	componentsByName = new Hashtable();
	Label clabel = new Label("  ");
	add("Center", clabel);
	Label slabel = new Label("Spiro v0.98 (c) 2000 by Neal Ziring",Label.LEFT);
	slabel.setFont(new Font("Serif", Font.BOLD, 10));
	add("South", slabel);
	myDesignPanel = dp;
	listForPresets = listEm;
	DesignCanvas dc = dp.getCanvas();
	dc.addMouseListener(this);
	dc.addMouseMotionListener(this);
	canvasSize = dc.getSize();
	if (canvasSize.width < 600) {
	    smFontSize = 10;
	    lgFontSize = 12;
	}
	innerPanel.setLayout(new GridLayout(0, 1, 1, 2));
	innerPanel.setBackground(Color.lightGray);
	smallFont = new Font("SansSerif", Font.PLAIN, smFontSize);
	smallBold = new Font("SansSerif", Font.BOLD, smFontSize);
	mediumFont = new Font("SansSerif", Font.PLAIN, lgFontSize);
	innerPanel.setFont(smallFont);

	// init the presets table
	initializePresets();

	// add components to the inner panel
	Panel pnl; TextField field;
	Label caption, cap2;  Button btn;  Choice ch; Checkbox cb, cb2;

	caption = new Label("Design center point:",Label.LEFT);
	caption.setFont(smallBold);
	innerPanel.add(caption);
	pnl = new Panel();  pnl.setLayout(new FlowLayout(FlowLayout.LEFT,1,1));
	caption = new Label("  X:",Label.RIGHT);  pnl.add(caption);
	field = new TextField("0.0", 6); pnl.add(field);
	new ControlItem(CENTERX, field, true, 
			-canvasSizeWorld, canvasSizeWorld, true);
	caption = new Label("  Y:",Label.RIGHT);  pnl.add(caption);
	field = new TextField("0.0", 6); pnl.add(field);
	new ControlItem(CENTERY, field, true, 
			-canvasSizeWorld, canvasSizeWorld, true);
	innerPanel.add(pnl);

	caption = new Label("Parameters for Stabile:",Label.LEFT);
	caption.setFont(smallBold);
	innerPanel.add(caption);
	pnl = new Panel();  pnl.setLayout(new FlowLayout(FlowLayout.LEFT,1,1));
	caption = new Label("  ");  pnl.add(caption);
	ch = new Choice(); 
	ch.add("Circle"); ch.add("Horiz. Bar"); ch.add("Vert. Bar");
	pnl.add(ch);
	new ControlItem(STABILETYPE, ch, false, 0.0, 1.0, false);
	caption = new Label("of size:",Label.CENTER); pnl.add(caption);
	field = new TextField("0.0", 7); pnl.add(field);
	new ControlItem(STABILESIZE, field, true, 
			canvasSizeWorld/40.0, canvasSizeWorld, true);
	innerPanel.add(pnl);

	caption = new Label("Parameters for Mobile:",Label.LEFT);
	caption.setFont(smallBold);
	innerPanel.add(caption);
	pnl = new Panel();  pnl.setLayout(new FlowLayout(FlowLayout.LEFT,1,1));
	caption = new Label("  Size:",Label.RIGHT);  pnl.add(caption);
	field = new TextField("0.0", 5); pnl.add(field);
	new ControlItem(MOBILESIZE, field, true, 
			canvasSizeWorld/50.0, canvasSizeWorld/1.8, true);
	caption = new Label("  Pen:",Label.RIGHT); pnl.add(caption);
	field = new TextField("0.0", 5); pnl.add(field);
	new ControlItem(PENSIZE, field, true, 
			0.0, canvasSizeWorld/1.5, true);
	innerPanel.add(pnl);

	pnl = new Panel();  pnl.setLayout(new FlowLayout(FlowLayout.LEFT,1,1));
	caption = new Label("  Spin",Label.RIGHT); pnl.add(caption);
	field = new TextField("40", 3); pnl.add(field);
	new ControlItem(REVOLUTIONS, field, true, 1.0, 300.0, false);
	caption = new Label(" times on"); pnl.add(caption);
	ch = new Choice();
	ch.addItem("Outside");
	ch.addItem("~Inside");
	ch.addItem("Inside");
	new ControlItem(SIDEMODE, ch, false, 0.0, 0.0, false);
	pnl.add(ch);
	innerPanel.add(pnl);

	caption = new Label("Pen size and color:",Label.LEFT);
	caption.setFont(smallBold);
	innerPanel.add(caption);
	pnl = new Panel();  pnl.setLayout(new FlowLayout(FlowLayout.LEFT,1,1));
	caption = new Label("  Solid:",Label.RIGHT);  pnl.add(caption);
	field = new TextField("1.0", 5); pnl.add(field);
	new ControlItem(PENSOLIDDIAMETER, field, true, 0.0, 8.0, false);
	caption = new Label("  Fade:",Label.RIGHT);  pnl.add(caption);
	field = new TextField("1.33", 5); pnl.add(field);
	new ControlItem(PENFADEDIAMETER, field, true, 0.1, 16.0, false);
	innerPanel.add(pnl);
	pnl = new Panel();  pnl.setLayout(new FlowLayout(FlowLayout.LEFT,1,1));
	caption = new Label("  Pen color: ",Label.RIGHT); pnl.add(caption);
	caption = new Label("      ");  caption.setBackground(Color.blue);
	pnl.add(caption);
	cap2 = new Label(" "); pnl.add(cap2);
	new ControlItem(PENCOLOR, caption, false, 0.0, 0.0, false);
	penColorPicker = new ColorPicker(parent, caption);
	penColorPicker.setLocation(120,120);
	penColorPicker.setTitle("Set pen color");
	btn = new Button("Set pen.."); pnl.add(btn);
	btn.setActionCommand(PENCOLOR); btn.addActionListener(this);
	innerPanel.add(pnl);

	caption = new Label("Advanced Parameters:",Label.LEFT);
	caption.setFont(smallBold);
	innerPanel.add(caption);
	pnl = new Panel();  pnl.setLayout(new FlowLayout(FlowLayout.LEFT,1,1));
	caption = new Label("  Dots/cycle:",Label.RIGHT); pnl.add(caption);
	field = new TextField("280", 4); pnl.add(field);
	new ControlItem(DOTSPER, field, true, 8.0, 1440.0, false);
	caption = new Label(" "); pnl.add(caption);
	cb = new Checkbox("adaptive", false); pnl.add(cb);
	new ControlItem(ADAPTIVE, cb, false, 0.0, 0.0, false);
	innerPanel.add(pnl);

	/*
	pnl = new Panel();  pnl.setLayout(new FlowLayout(FlowLayout.LEFT,1,1));
	caption = new Label("  Paper color:",Label.RIGHT); pnl.add(caption);
	caption = new Label("      ");  caption.setBackground(Color.white);
	pnl.add(caption);
	cap2 = new Label(" "); pnl.add(cap2);
	new ControlItem(PAPERCOLOR, caption, false, 0.0, 0.0, false);
	paperColorPicker = new ColorPicker(parent, caption);
	btn = new Button("Pick.."); pnl.add(btn);
	btn.setActionCommand(PAPERCOLOR); btn.addActionListener(this);
	innerPanel.add(pnl);
	*/

	caption = new Label("Preset designs:",Label.LEFT);
	caption.setFont(smallBold);
	innerPanel.add(caption);
	pnl = new Panel();  pnl.setLayout(new FlowLayout(FlowLayout.LEFT,1,1));
	caption = new Label("  "); pnl.add(caption);
	presets = new Choice();
	for(int pc = 0; pc < rawpresets.length; pc += 2) {
	    presets.addItem(rawpresets[pc][0]);
	}
	pnl.add(presets);
	presets.addItemListener(this);
	innerPanel.add(pnl);

	caption = new Label("-------------------", Label.CENTER);
	innerPanel.add(caption);

	Button renderBtn = new Button("Draw this Design!");
	renderBtn.setFont(mediumFont);
	renderBtn.setActionCommand(RENDER);
	innerPanel.add(renderBtn);
	pnl = new Panel();  pnl.setLayout(new GridLayout(1,2));
	pnl.setFont(mediumFont);
	Button helpBtn = new Button("Help");
	helpBtn.setActionCommand(HELP); pnl.add(helpBtn);
	Button closeBtn = new Button("Close");
	closeBtn.setActionCommand(CLOSE); pnl.add(closeBtn);
	innerPanel.add(pnl);
	add("North", innerPanel);

	// set up listeners
	renderBtn.addActionListener(this);
	helpBtn.addActionListener(this);
	closeBtn.addActionListener(this);
	Enumeration ec;
	for(ec = components.keys(); ec.hasMoreElements(); ) {
	    Component c1 = (Component)(ec.nextElement());
	    if (c1 instanceof TextField) {
		((TextField)c1).addTextListener(this);
	    }
	}

	// apply first preset
	applyPreset(0);

	// do window listening
	parent.addWindowListener(this);
    }

    /**
     * Initialize the presets.  Each preset is represented here
     * in the constant pool of this class as an array of arrays
     * of strings, in pairs.  The odd elements are singleton
     * arrays that contain the name of the preset.
     * The even elements are even-length arrays containing 
     * name-value pairs: names of control items and the value
     * of them for the preset.  All size and dimension values
     * in the presets list should be created as if for a 
     * world extent of 10.0.  
     *
     * At present, initializePresets does nothing.
     */

    protected static String [][] rawpresets = {
	{ "Classic" },
	{   CENTERX, "0.0",
	    CENTERY, "0.0",
	    STABILETYPE, "0",
	    STABILESIZE, "7.8125",
	    MOBILESIZE, "3.4375",
	    PENSIZE, "2.15625",
	    REVOLUTIONS, "40.0",
	    SIDEMODE, "2",
	    PENSOLIDDIAMETER, "1.0",
	    PENFADEDIAMETER, "1.5",
	    DOTSPER, "330.0",
	    PENCOLOR, "0x06600CC",
	    ADAPTIVE, "1" },
	{ "Simple 1" }, 
	{ CENTERX, "0.0", CENTERY, "0.0",
	  STABILETYPE, "0", STABILESIZE, "4.0",
	  MOBILESIZE, "1.1111", PENSIZE, "1.0",
	  REVOLUTIONS, "50", SIDEMODE, "0",
	  PENSOLIDDIAMETER, "1.0", PENFADEDIAMETER, "1.5",
	  PENCOLOR, "3203104", DOTSPER, "400"
	},
	{ "Fancy 1" },
	{ CENTERX, "0.0", CENTERY, "0.0",
	  STABILETYPE, "0", STABILESIZE, "6.95",
	  MOBILESIZE, "2.01", PENSIZE, "2.5",
	  REVOLUTIONS, "90", SIDEMODE, "2",
	  ADAPTIVE, "true",
	  PENSOLIDDIAMETER, "0.8", PENFADEDIAMETER, "1.3",
	  PENCOLOR, "0x0033FF", DOTSPER, "330"
	},
	{ "Test 2" },
	{ CENTERX, "0.0", CENTERY, "0.0",
	  STABILETYPE, "0", STABILESIZE, "4",
	  MOBILESIZE, "1.0", PENSIZE, "1.0",
	  REVOLUTIONS, "4", SIDEMODE, "0",
	  ADAPTIVE, "true",
	  PENSOLIDDIAMETER, "2", PENFADEDIAMETER, "3",
	  PENCOLOR, "0x00333", DOTSPER, "15"
	},
	{ "Nicer ?" },
        {   CENTERX, "0.0",
	    CENTERY, "0.0",
	    STABILETYPE, "0",
	    STABILESIZE, "5.0",
	    MOBILESIZE, "1.1",
	    PENSIZE, "1.3",
	    REVOLUTIONS, "50.0",
	    SIDEMODE, "2",
	    PENSOLIDDIAMETER, "2.0",
	    PENFADEDIAMETER, "3.5",
	    DOTSPER, "400.0",
	    ADAPTIVE, "1" 
	},
	{ "Lacy 1" },
	{   CENTERX, "0.0",
	    CENTERY, "0.0",
	    STABILETYPE, "0",
	    STABILESIZE, "4.88",
	    MOBILESIZE, "0.84",
	    PENSIZE, "0.96",
	    REVOLUTIONS, "144.0",
	    SIDEMODE, "0",
	    PENSOLIDDIAMETER, "0.3",
	    PENFADEDIAMETER, "2.0",
	    PENCOLOR, "51",
	    DOTSPER, "360.0",
	    ADAPTIVE, "false" 
	},
	{ "Lacy 2" },
	{   CENTERX, "0.0",
	    CENTERY, "0.0",
	    STABILETYPE, "0",
	    STABILESIZE, "8.4",
	    MOBILESIZE, "1.155",
	    PENSIZE, "1.0",
	    REVOLUTIONS, "80.0",
	    SIDEMODE, "2",
	    PENSOLIDDIAMETER, "1.0",
	    PENFADEDIAMETER, "2.5",
	    DOTSPER, "400.0",
	    ADAPTIVE, "1" },
	{"PowderPuff 1" },
	{   CENTERX, "0.0",
	    CENTERY, "0.0",
	    STABILETYPE, "0",
	    STABILESIZE, "6.9375",
	    MOBILESIZE, "2.234375",
	    PENSIZE, "2.53125",
	    REVOLUTIONS, "82.0",
	    SIDEMODE, "1",
	    PENSOLIDDIAMETER, "1.0",
	    PENFADEDIAMETER, "1.5",
	    DOTSPER, "400.0",
	    ADAPTIVE, "0" },
	{ "Basket?" },
	{   CENTERX, "0.0",
	    CENTERY, "0.0",
	    STABILETYPE, "0",
	    STABILESIZE, "1.5625",
	    MOBILESIZE, "3.046875",
	    PENSIZE, "3.125",
	    REVOLUTIONS, "9.0",
	    SIDEMODE, "0",
	    PENSOLIDDIAMETER, "0.55",
	    PENFADEDIAMETER, "1.9",
	    DOTSPER, "333.0",
	    ADAPTIVE, "1" },
	{ "Star 1" },
	{   CENTERX, "0.0",
	    CENTERY, "0.0",
	    STABILETYPE, "0",
	    STABILESIZE, "10.3125",
	    MOBILESIZE, "4.140625",
	    PENSIZE, "1.53125",
	    REVOLUTIONS, "30.0",
	    SIDEMODE, "2",
	    PENSOLIDDIAMETER, "0.5",
	    PENFADEDIAMETER, "3.5",
	    DOTSPER, "500.0",
	    PENCOLOR, "0x0CCffff",
	    ADAPTIVE, "1" },
	{ "String Border" },
	{   CENTERX, "0.0",
	    CENTERY, "0.0",
	    STABILETYPE, "0",
	    STABILESIZE, "16.25",
	    MOBILESIZE, "4.265625",
	    PENSIZE, "3.75",
	    REVOLUTIONS, "90.0",
	    SIDEMODE, "2",
	    PENSOLIDDIAMETER, "0.5",
	    PENFADEDIAMETER, "1.5",
	    PENCOLOR, "0x0663300",
	    DOTSPER, "450.0",
	    ADAPTIVE, "1" },
	{ "Delicate3" },
	{   CENTERX, "0.0",
	    CENTERY, "0.09375",
	    STABILETYPE, "0",
	    STABILESIZE, "8.125",
	    MOBILESIZE, "1.125",
	    PENSIZE, "1.0",
	    REVOLUTIONS, "90.0",
	    SIDEMODE, "2",
	    PENSOLIDDIAMETER, "1.0",
	    PENFADEDIAMETER, "1.25",
	    DOTSPER, "90.0",
	    PENCOLOR, "0x0CC33FF",
	    ADAPTIVE, "0" },
	{"Border Bottom" },
	{   CENTERX, "0.0",
	    CENTERY, "9.95",
	    STABILETYPE, "1",
	    STABILESIZE, "8.125",
	    MOBILESIZE, "1.125",
	    PENSIZE, "1.0",
	    REVOLUTIONS, "90.0",
	    SIDEMODE, "2",
	    PENSOLIDDIAMETER, "1.0",
	    PENFADEDIAMETER, "1.25",
	    DOTSPER, "90.0",
	    ADAPTIVE, "1" },
	{ "Border Left" },
	{   CENTERX, "-9.95",
	    CENTERY, "0.0",
	    STABILETYPE, "2",
	    STABILESIZE, "8.125",
	    MOBILESIZE, "1.125",
	    PENSIZE, "1.0",
	    REVOLUTIONS, "90.0",
	    SIDEMODE, "2",
	    PENSOLIDDIAMETER, "1.0",
	    PENFADEDIAMETER, "1.25",
	    DOTSPER, "90.0",
	    ADAPTIVE, "1" },
	{ "Border Right" },
	{   CENTERX, "10.0",
	    CENTERY, "0.0",
	    STABILETYPE, "2",
	    STABILESIZE, "8.125",
	    MOBILESIZE, "1.125",
	    PENSIZE, "1.0",
	    REVOLUTIONS, "90.0",
	    SIDEMODE, "2",
	    PENSOLIDDIAMETER, "1.0",
	    PENFADEDIAMETER, "1.25",
	    DOTSPER, "90.0",
	    ADAPTIVE, "1" },
	{"Border Top" },
	{   CENTERX, "0.0",
	    CENTERY, "-9.95",
	    STABILETYPE, "1",
	    STABILESIZE, "8.125",
	    MOBILESIZE, "1.125",
	    PENSIZE, "1.0",
	    REVOLUTIONS, "90.0",
	    SIDEMODE, "2",
	    PENSOLIDDIAMETER, "1.0",
	    PENFADEDIAMETER, "1.25",
	    DOTSPER, "90.0",
	    ADAPTIVE, "1" },
	{ "Puffy Cloud" },
	{   CENTERX, "0.0",
	    CENTERY, "0.0",
	    STABILETYPE, "0",
	    STABILESIZE, "9.375",
	    MOBILESIZE, "0.9375",
	    PENSIZE, "0.9375",
	    REVOLUTIONS, "51.0",
	    SIDEMODE, "1",
	    PENSOLIDDIAMETER, "0.15",
	    PENFADEDIAMETER, "8.9",
	    DOTSPER, "900.0",
	    ADAPTIVE, "1" },
	{ "Wirey Pt1" },
	{ CENTERX, "0.0",
	  CENTERY, "0.0",
	  STABILETYPE, "0",
	  STABILESIZE, "8.75",
	  MOBILESIZE, "2.40625",
	  PENSIZE, "1.84375",
	  REVOLUTIONS, "40.0",
	  SIDEMODE, "2",
	  PENSOLIDDIAMETER, "1.0",
	  PENFADEDIAMETER, "1.5",
	  PENCOLOR, "10066278",
	  DOTSPER, "330.0",
	  ADAPTIVE, "1" },
	{ "Wirey Pt 2" },
	{ CENTERX, "0.0",
	  CENTERY, "0.0",
	  STABILETYPE, "0",
	  STABILESIZE, "4.375",
	  MOBILESIZE, "1.203125",
	  PENSIZE, "0.921875",
	  REVOLUTIONS, "40.0",
	  SIDEMODE, "1",
	  PENSOLIDDIAMETER, "1.0",
	  PENFADEDIAMETER, "1.5",
	  PENCOLOR, "10066278",
	  DOTSPER, "330.0",
	  ADAPTIVE, "1" },
	{ "Seventeen" },
	{ CENTERX, "0.0",
	  CENTERY, "0.0",
	  STABILETYPE, "0",
	  STABILESIZE, "3.1875",
	  MOBILESIZE, "2.25",
	  PENSIZE, "3.1875",
	  REVOLUTIONS, "17.1",
	  SIDEMODE, "0",
	  PENSOLIDDIAMETER, "1.25",
	  PENFADEDIAMETER, "2.95",
	  PENCOLOR, "10092492",
	  DOTSPER, "330.0",
	  ADAPTIVE, "1" },
    };

    public void initializePresets() {
	return;
    }

    public void applyPreset(int index) {
	String pn;
	String [] val;
	String cn, cv;
	pn = rawpresets[index * 2][0];
	val = rawpresets[index * 2 + 1];
	myDesignPanel.setStatus("Using preset " + pn);
	int i;
	for(i = 0; i < val.length; i += 2) {
	    cn = val[i]; cv = val[i + 1];
	    ControlItem ci = getItemByName(cn);
	    if (ci != null) {
		if (ci.comp instanceof TextField) {
		    try {
			double dv = Double.valueOf(cv).doubleValue();
			if (ci.needsScale) {
			    dv = (dv / BASE_CSW) * (canvasSizeWorld/2.0);
			}
			((TextField)(ci.comp)).setText("" + dv);
		    }
		    catch (NumberFormatException nfe) { }
		}
		else if (ci.comp instanceof Label) {
		    int colvalue = 0;
		    try {
			if (cv.startsWith("0x")) {
			    colvalue = Integer.parseInt(cv.substring(2),16);
			}
			else {
			    colvalue = Integer.parseInt(cv);
			}
		    } catch (NumberFormatException nfe) { }
		    Color col = new Color(colvalue);
		    ci.comp.setBackground(col);
		}
		else if (ci.comp instanceof Choice) {
		    int indx = 0;
		    try { 
			indx = Integer.parseInt(cv); 
			((Choice)(ci.comp)).select(indx);
		    }
		    catch (Exception nfe) { }
		}
		else if (ci.comp instanceof Checkbox) {
		    boolean bval = false;
		    try {
			if (cv.length() >= 4) {
			    bval = Boolean.valueOf(cv).booleanValue();
			}
			else {
			    bval = (Integer.parseInt(cv) != 0);
			}
		    } catch (Exception e) { }
		    ((Checkbox)(ci.comp)).setState(bval);
		}
	    }
	}
    }
	


    public void doDesign() {
	double cx, cy;
	int whichstabile;
	double ssize, msize, psize;
	double psr, pfr;
	double revs;
	int sidemode;
	boolean adaptive;
	double pensolid, penfade;
	double dotsper;
	
	ControlItem ci;
	cx = getItemByName(CENTERX).getDoubleValue();
	cy = getItemByName(CENTERY).getDoubleValue();

	ci = getItemByName(STABILETYPE);
	whichstabile = ((Choice)(ci.comp)).getSelectedIndex();
	ssize = getItemByName(STABILESIZE).getDoubleValue();
	msize = getItemByName(MOBILESIZE).getDoubleValue();
	psize = getItemByName(PENSIZE).getDoubleValue();
	revs = getItemByName(REVOLUTIONS).getDoubleValue();
	ci = getItemByName(SIDEMODE);
	sidemode = ((Choice)(ci.comp)).getSelectedIndex();
	pensolid = getItemByName(PENSOLIDDIAMETER).getDoubleValue();
	penfade = getItemByName(PENFADEDIAMETER).getDoubleValue();
	ci = getItemByName(PENCOLOR);
	penColor = ci.comp.getBackground();
	dotsper = getItemByName(DOTSPER).getDoubleValue();
	ci = getItemByName(ADAPTIVE);
	adaptive = ((Checkbox)(ci.comp)).getState();
	
	if (listForPresets) {
	    double f = BASE_CSW / (canvasSizeWorld / 2.0);
	    System.out.println("  { \"SomeName\" },");
	    System.out.println("  { CENTERX, \"" + cx*f + "\",");
	    System.out.println("    CENTERY, \"" + cy*f + "\",");
	    System.out.println("    STABILETYPE, \"" + whichstabile + "\",");
	    System.out.println("    STABILESIZE, \"" + ssize*f + "\",");
	    System.out.println("    MOBILESIZE, \"" + msize*f + "\",");
	    System.out.println("    PENSIZE, \"" + psize*f + "\",");
	    System.out.println("    REVOLUTIONS, \"" + revs + "\",");
	    System.out.println("    SIDEMODE, \"" + sidemode + "\",");
	    System.out.println("    PENSOLIDDIAMETER, \"" + pensolid  + "\",");
	    System.out.println("    PENFADEDIAMETER, \"" + penfade + "\",");
	    System.out.println("    PENCOLOR, \"" + 
			       (((((penColor.getRed())<<8)|penColor.getGreen())<<8)|penColor.getBlue()) + "\",");
	    System.out.println("    DOTSPER, \"" + dotsper + "\",");
	    System.out.println("    ADAPTIVE, \"" + (adaptive?1:0) + "\" },");
	}

	Stabile s;
	Mobile m;

	switch (whichstabile) {
	case 0:
	    s = new CircleStabile(new PPoint(cx,cy), ssize);
	    break;
	case 1:
	    s = new BarStabile(new PPoint(cx,cy), ssize);
	    break;
	case 2:
	    s = new PylonStabile(new PPoint(cx,cy), ssize);
	    break;
	default:
	    s = new CircleStabile(new PPoint(cx,cy), ssize);
	    break;
	}

	m = new CircleMobile(msize, psize);
	double ia = 0.0;
	Design des = new Design(m, s, ia, revs, canvasSizeWorld,
				canvasSize.width, pensolid, penfade,
				(int)dotsper, sidemode);
	des.addObserver(myDesignPanel);
	des.setAdaptive(adaptive);
	des.addObserver(this);
	Thread t1 = new Thread(des);
	t1.start();
    }
	
    public void update(Observable o, Object arg) {
	if ((o instanceof Design) && (arg instanceof String)) {
	    Image i = ((Design)o).getImage(myDesignPanel.getCanvas(), 
					   penColor,Color.white,true);
	    myDesignPanel.getCanvas().addImage(i, (Design)o);
	    ((Design)o).release();
	    ((Design)o).deleteObserver(this);
	}
    }

    public void doColorPicker(Dialog cp) {
	cp.setModal(true);
	cp.show();
    }

    public void actionPerformed(ActionEvent e) {
	String act = e.getActionCommand();
	if (act.equals(PENCOLOR) || act.equals(PENCOLORPICK)) {
	    penColorPicker.setModal(true);
	    penColorPicker.show();
	}
	else if (act.equals(CLOSE)) {
	    closeUpShop();
	}
	else if (act.equals(HELP)) {
	    AppletContext ac;
	    if (parentApplet != null) {
		ac = parentApplet.getAppletContext();
		String helpStr;
		helpStr = parentApplet.getParameter("help");
		if (helpStr == null) 
		    helpStr = parentApplet.getParameter("helpURL");
		if (helpStr == null) 
		    helpStr = parentApplet.getParameter("helpPage");
		if (helpStr != null) {
		    URL base = parentApplet.getDocumentBase();
		    try {
			URL helpURL = new URL(base, helpStr);
			ac.showDocument(helpURL, "help");
		    } catch (Exception e2) {
			myDesignPanel.setStatus("Could not load help page: " + helpStr);
		    }
		}
	    }
	}
	else if (act.equals(RENDER)) {
	    String inv = validateAllItems();
	    if (inv != null) {
		myDesignPanel.setStatus("Item(s) with invalid values: " + inv);
	    }
	    else {
		doDesign();
	    }
	}
    }

    public void closeUpShop() {
	parentFrame.setVisible(false);
	parentFrame.dispose();
	if (parentApplet == null)
	    System.exit(0);
	else {
	    if (parentApplet instanceof SpiroApplet) {
		((SpiroApplet)parentApplet).canvasesUp -= 1;
	    }
	}
    }

    public String validateAllItems() {
	StringBuffer sb = new StringBuffer();
	int invCount = 0;
	Enumeration e;
	Component c; 
	for(e = components.keys(); e.hasMoreElements(); ) {
	    c = (Component)(e.nextElement());
	    if (!validateItem(c)) {
		if (invCount > 0) sb.append(", ");
		sb.append(getItem(c).name);
		invCount++;
	    }
	}
	if (invCount == 0)
	    return null;
	else
	    return sb.toString();
    }

    public void textValueChanged(TextEvent e) {
	validateItem((Component)(e.getSource()));
    }

    public void itemStateChanged(ItemEvent e) {
	if (e.getSource() == presets) {
	    int p = presets.getSelectedIndex();
	    applyPreset(p);
	}
    }

    protected double startDx = 0.0, startDy = 0.0;

    public void setCenter(int px, int py, boolean start) {
	double dx, dy;
	dx = ((px / (1.0 * canvasSize.width)) * canvasSizeWorld) - 
	    (canvasSizeWorld / 2.0);
	dy = ((py / (1.0 * canvasSize.height)) * canvasSizeWorld) - 
	    (canvasSizeWorld / 2.0);
	if (start) {
	    startDx = dx;  startDy = dy;
	}
	ControlItem ci = getItemByName(CENTERX);
	if (ci != null) {
	    ((TextField)(ci.comp)).setText("" + Math.rint(dx));
	}
	ci = getItemByName(CENTERY);
	if (ci != null) {
	    ((TextField)(ci.comp)).setText("" + Math.rint(dy));
	}
    }

    public void setSize(int px, int py) {
	double dx, dy;
	dx = ((px / (1.0 * canvasSize.width)) * canvasSizeWorld) - 
	    (canvasSizeWorld / 2.0);
	dy = ((py / (1.0 * canvasSize.height)) * canvasSizeWorld) - 
	    (canvasSizeWorld / 2.0);
	double sz, deltax, deltay;
	deltax = startDx - dx;
	deltay = startDy - dy;
	sz = Math.rint(Math.sqrt(deltax * deltax + deltay * deltay));
	ControlItem ci = getItemByName(STABILESIZE);
	if (ci != null) {
	    ((TextField)(ci.comp)).setText("" + sz);
	}
    }	

    public void mouseDragged(MouseEvent e) {
	if (e.isShiftDown()) {
	    setSize(e.getX(), e.getY());
	}
	else {
	    setCenter(e.getX(),e.getY(), true);
	}
    }
    public void mouseMoved(MouseEvent e) {}
    public void mouseClicked(MouseEvent e) {}
    public void mousePressed(MouseEvent e) {
	setCenter(e.getX(),e.getY(), true);
    }
    public void mouseReleased(MouseEvent e) {
	if (e.isShiftDown()) {
	    setSize(e.getX(), e.getY());
	}
	else {
	    setCenter(e.getX(),e.getY(), false);
	}
    }
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}

    public void windowOpened(WindowEvent e) { }
    public void windowClosing(WindowEvent e) {
	closeUpShop();
    }
    public void windowClosed(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowActivated(WindowEvent e) {}
    public void windowDeactivated(WindowEvent e) {}

    /***
     * Main for Testing...
    static Frame f;
    static DesignPanel dp;
    static ControlPanel cp;

    public static void main(String [] args) {
	int siz = 768;
	if (args.length == 0) {
	    System.out.println("Usage: java ControlPanel size-in-pixels");
	    System.exit(0);
	}
	if (args.length > 0) {
	    siz = Integer.parseInt(args[0]);
	}
	if (siz < 512 || siz > 1600) siz = 768;

	f = new Frame("Test of ControlPanel");
	dp = new DesignPanel(siz, new FileSaveManager());
	f.add("Center", dp);
	cp = new ControlPanel(dp, (double)siz, f, null, true);
	f.add("West", cp);

	f.pack();
	f.show();
    }

    */

}

