import java.awt.*;
import java.awt.image.*;
import java.util.*;
import util.*;

/**
 * An instance of this class represents a single
 * spirograph design.  It encapsulates the
 * notion of using the stabile and the mobile
 * to create the design and render it onto an
 * ImageBlotter.
 *
 * When executing a design, this class calls
 * its observers for every 2% completed.
 */

public class Design extends Observable implements Runnable 
{
    public static final int OUTSIDE_MODE  = 0;
    public static final int INSIDE_CONTRA_MODE = 1;
    public static final int INSIDE_MODE   = 2;

    ImageBlotter	blotter;
    ImageBlot		blot;
    Mobile		m;
    Stabile		s;

    double		initAngle;
    double		maxAngle;

    Color		fgColor;
    Color		bgColor;
    
    PPoint		center;
    double		canvasSizeW;
    int			canvasSizePixels;
    int			sideMode;
    boolean		adaptive;
    double		angleQuantum;

    int			stepCount;
    boolean		completed;
    int			blotCount;

    protected void init(Mobile _m, Stabile _s,
		   double ia, double ma, 
		   double csw, int cs,
		   double penSolidRadius, 
		   double penTotalRadius, 
		   double qa, int sm)
    {
	completed = false;
	adaptive = false;

	m = _m;
	s = _s;
	initAngle = ia;
	maxAngle = ma;
	sideMode = sm;
	canvasSizeW = csw;
	canvasSizePixels = cs;
	center = s.getCenter();

	// create blot and blotter
	blot = new ImageBlot(penSolidRadius, penTotalRadius);
	blotter = new ImageBlotter(canvasSizeW/2.0,
				   canvasSizeW/2.0,
				   cs, cs,
				   Color.black, Color.white,
				   blot);

	// set other parameters
	angleQuantum = qa;
    }

    public Design(Mobile _m, Stabile _s, double ia, 
		  double revolutions, double csw,
		  int cs, double penSolidSize, double penTotalSize,
		  int stepsPer, int side_mode)
    {
	double qa;
	qa = (2.0 * Math.PI) / stepsPer;
	init(_m, _s, ia, ia+(Math.PI * 2.0 * revolutions), csw, cs,
	     penSolidSize / 2.0, penTotalSize / 2.0, qa, side_mode);
    }

    public Design(int mdiameterPixels, 
		  int penPercentRadius,
		  int sdiameterPixels,
		  int revolutions, int cs,
		  Point center, int penSize, boolean isInside)
    {
	double mradius = mdiameterPixels / 2.0;
	double mpradius = mradius * (penPercentRadius / 100.0);
	double sradius = sdiameterPixels / 2.0;
	PPoint ppc = new PPoint(center.x - (cs / 2.0),
				center.y - (cs / 2.0));
	double ma = revolutions * (2.0 * Math.PI);
	
	CircleMobile _m = 
	    new CircleMobile(mradius, mpradius);
	CircleStabile _s =
	    new CircleStabile(ppc, sradius);

	double penSolidRadius, penTotalRadius;
	penSolidRadius = penSize / 2.0;
	penTotalRadius = 1.5 * penSolidRadius;

	double qa;
	qa = Math.PI / (180.0 / penSize);

	// call init
	init(_m, _s, 0.0, ma, cs * 1.0, cs, 
	     penSolidRadius, penTotalRadius, qa, 
	     (isInside?INSIDE_MODE:OUTSIDE_MODE));
    }
		  
    public void setAdaptive(boolean ad) { adaptive = ad; }
    public boolean isAdaptive() { return adaptive; }

    public boolean isCompleted() { return completed; }

    public void run() {
	try {
	long startTime, endTime;
	blotCount = 0;
	startTime = System.currentTimeMillis();
	PPoint pen = new PPoint();
	PPoint prev = new PPoint();
	stepCount = (int)Math.rint((maxAngle - initAngle) / angleQuantum + 0.5);
	int step, obsCount;
	obsCount = stepCount / 50;
	if (obsCount < 1) obsCount = 1;
	for(step = 0; step < stepCount; ) {
	    doStep(step, pen, prev);
	    // notify observers if necessary yet
	    step += 1;
	    if ((step % obsCount) == 0) {
		message(new Integer(step));
	    }
	}
	completed = true;
	endTime = System.currentTimeMillis();
        message("Done (" +
		(endTime - startTime) + "ms. elapsed time, " +
		+ blotCount + " points applied.)");
	}
	catch (OutOfMemoryError err) {
	    blotter.release();
	    System.gc();
	    message("Error: ran out of memory.");
	}
	return;
    }

    private final void doStep(int step, PPoint pen, PPoint prev) {
	double a, d1, n1, d2;
	boolean isIn, isntOut;
	PPoint cp;
	isIn = (sideMode == INSIDE_MODE);
	isntOut = (sideMode != OUTSIDE_MODE);
	if (!adaptive || step == 0) {
	    a = initAngle + (step * angleQuantum);
	    d1 = m.angleToDistance(a);
	    if (isIn) d2 = -d1; else d2 = d1;
	    n1 = s.distanceToNormal(d2, isntOut);
	    cp = s.distanceToPoint(d2);
	    m.findPen(cp, n1, d1, isIn, pen);
	    blotter.applyBlot(pen.x, pen.y);
	    blotCount++;
	    prev.x = pen.x; prev.y = pen.y;
	}
	else {
	    double aq = angleQuantum;
	    double pdist = 0.0;
	    double lim = blotter.inverseTransformWidth(blot.fgrad + blot.bgrad);
	    // find a reasonably smooth angle quantum (max divisor, 8)
	    int cnt = 1;
	    do {
		a = initAngle + ((((step - 1) * cnt) + 1) * aq);
		d1 = m.angleToDistance(a);
		if (isIn) d2 = -d1; else d2 = d1;
		n1 = s.distanceToNormal(d2, isntOut);
		cp = s.distanceToPoint(d2);
		m.findPen(cp, n1, d1, isIn, pen);
		// ORIG: d1 = m.angleToDistance(a);
		// ORIG: n1 = s.distanceToNormal(d1,inside);
		// ORIG: cp = s.distanceToPoint(d1);
		// ORIG: m.findPen(cp, n1, d1, false, pen);
		// ORIG: m.findPen(cp, n1, d1, pen);
		pdist = pen.distanceTo(prev);
		if (pdist > lim) { cnt *= 2; aq /= 2.0; }
	    } while (pdist > lim && cnt < 8);
	    if (cnt == 1) {
		blotter.applyBlot(pen.x, pen.y);
		prev.x = pen.x; prev.y = pen.y;
		blotCount++;
	    }
	    else {
		for(int i = 1; i <= cnt; i++) {
		    a = initAngle + ((((step - 1) * cnt) + i) * aq);
		    d1 = m.angleToDistance(a);
		    if (isIn) d2 = -d1; else d2 = d1;
		    n1 = s.distanceToNormal(d2, isntOut);
		    cp = s.distanceToPoint(d2);
		    m.findPen(cp, n1, d1, isIn, pen);
		    // ORIG: d1 = m.angleToDistance(a);
		    // ORIG: n1 = s.distanceToNormal(d1,inside);
		    // ORIG: cp = s.distanceToPoint(d1);
		    // ORIG: m.findPen(cp, n1, d1, false, pen);
		    // ORIG: m.findPen(cp, n1, d1, pen);
		    blotter.applyBlot(pen.x, pen.y);
		    blotCount++;
		    prev.x = pen.x; prev.y = pen.y;
		}
	    }
	}
	return;
    }

    public String formatMessage(Object arg) {
	if (arg instanceof Number) {
	    int steps = ((Number)arg).intValue();
	    StringBuffer sb = new StringBuffer();
	    sb.append(steps);
	    sb.append(" of ");
	    sb.append(stepCount);
	    sb.append(" (");
	    sb.append((int)((steps * 100)/stepCount));
	    sb.append("%)");
	    return sb.toString();
	}
	else {
	    return arg.toString();
	}
    }

    public Image getImage(Component c,
			  Color fgColor, Color bgColor, 
			  boolean useTrans)
    {
	blotter.setColors(fgColor, bgColor);
	return c.createImage(blotter.getProducer(null, useTrans));
    }

    public ImageBlotter getBlotter() { return blotter; }

    public void release() {
	blotter.release();
	return;
    }

    public void message(Object o) {
	setChanged(); notifyObservers(o);
    }
}


