import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import java.io.*;

public class DesignPanel extends Panel 
    implements ActionListener, Observer
{
    static final String ACTION_SAVE = "save";
    static final String ACTION_CLEAR= "clear";
    static final String ACTION_COLOR= "color";
    static final String ACTION_CLEAR_LAST= "last";

    DesignCanvas myCanvas;
    Panel        controlPane;
    Label	 statusLine;
    Button	 clearButton;
    Button	 clearLastButton;
    Button	 saveButton;
    Button	 colorButton;
    ColorPicker  paperColorPicker;
    SaveManager  saver;
    
    public DesignCanvas getCanvas() { return myCanvas; }

    public DesignPanel(int siz, SaveManager mgr) {
	saver = mgr;
	if (saver != null) saver.addObserver(this);

	setLayout(new BorderLayout(4,4));
	if (siz < 100 || siz > 2048)
	    throw new IllegalArgumentException("bad size");
	myCanvas = new DesignCanvas(siz);
	myCanvas.setBackground(Color.white);
	controlPane = new Panel();
	controlPane.setLayout(new FlowLayout(FlowLayout.LEFT));
	statusLine = new Label("Spiro Design Canvas",
			       Label.LEFT);
	statusLine.setFont(new Font("SansSerif", Font.PLAIN, 12));
	statusLine.setBackground(Color.lightGray);

	clearButton = new Button("Clear");
	clearButton.setActionCommand(ACTION_CLEAR);
	clearButton.addActionListener(this);

	colorButton = new Button("Set background..");
	colorButton.setActionCommand(ACTION_COLOR);
	colorButton.addActionListener(this);

	saveButton = new Button("Save Image");
	saveButton.setActionCommand(ACTION_SAVE);
	saveButton.addActionListener(this);
	if (saver == null) saveButton.setEnabled(false);

	clearLastButton = new Button("Clear Last");
	clearLastButton.setActionCommand(ACTION_CLEAR_LAST);
	clearLastButton.addActionListener(this);

	controlPane.setFont(new Font("SansSerif", Font.PLAIN, 12));
	controlPane.add(new Label("Design operations:",Label.LEFT));
	controlPane.add(clearButton);
	controlPane.add(clearLastButton);
	controlPane.add(new Label(" "));
	controlPane.add(colorButton);
	controlPane.add(new Label(" "));
	controlPane.add(saveButton);
	controlPane.setBackground(Color.lightGray);

	paperColorPicker = null;

	add("North", controlPane);
	add("Center", myCanvas);
	add("South", statusLine);

	setBackground(Color.black);
    }

    public void setStatus(String s) { statusLine.setText(s); }

    public void actionPerformed(ActionEvent e) {
	String cmd = e.getActionCommand();
	if (cmd == null || cmd.length() == 0) return;

	if (cmd.equals(ACTION_SAVE)) {
	    // System.out.println("Background is: " + myCanvas.getBackground());
	    saver.performSave(myCanvas, myCanvas.getBackground());
	}
	else if (cmd.equals(ACTION_COLOR)) {
	    if (paperColorPicker == null) {
		paperColorPicker = new ColorPicker(this, myCanvas);
		paperColorPicker.setLocation(460,140);
		paperColorPicker.setTitle("Set background color");
	    }
	    paperColorPicker.setModal(true);
	    paperColorPicker.show();
	    myCanvas.repaint();
	}
	else if (cmd.equals(ACTION_CLEAR)) {
	    setStatus("Cleared all designs.");
	    myCanvas.removeAllImages();
	    myCanvas.repaint();
	}
	else if (cmd.equals(ACTION_CLEAR_LAST)) {
	    int ci = myCanvas.countImages();
	    if (ci > 0) {
		setStatus("Cleared design " + (ci + 1) + ".");
		myCanvas.removeImage(ci - 1);
	    }
	}
    }

    public void update(Observable o, Object arg) {
	if (o instanceof Design) {
	    setStatus(((Design)o).formatMessage(arg));
	}
	else if (o instanceof SaveManager) {
	    setStatus(((SaveManager)o).formatMessage(arg));
	}
	else if (arg != null) {
	    setStatus("Status: " + arg.toString());
	}
	else {
	    setStatus("Anomalous update from " + o.toString());
	}
	return;
    }

   /**
     * Main for testing..
     * args[0] = stabile radius
     * args[1] = mobile radius
     * args[2] = mobile pen radius
     * args[3] = inside(t) or outside (f or absent)
    static Frame f;
    static DesignPanel dp;
    static DesignCanvas dc;
    public static void main(String [] args) {
	if (args.length < 3) {
	    System.out.println("Usage: java DesignCanvas sradius mradius pradius true/false revs adaptSmooth");
	    System.exit(0);
	}

	double sradius = 0.0;
	double mradius = 0.0;
	double pradius = 0.0;
	int revs = 30;
	boolean inside = false;
	boolean adaptive = false;
	double canvasSizeWorld;
       
	try {
	    sradius = Double.valueOf(args[0]).doubleValue();
	    mradius = Double.valueOf(args[1]).doubleValue();
	    pradius = Double.valueOf(args[2]).doubleValue();
	    if (args.length > 3) 
		inside = Boolean.valueOf(args[3]).booleanValue();
	    if (args.length > 4) 
		revs = Integer.parseInt(args[4]);
	    if (args.length > 5) 
		adaptive = Boolean.valueOf(args[5]).booleanValue();
	}
	catch (NumberFormatException nfe) {
	    System.out.println("Bad number: " + nfe);
	    System.exit(0);
	}

	int siz = 640;
	f = new Frame("DesignPanel Test");
	FileSaveManager fsm = new FileSaveManager();
	dp = new DesignPanel(siz, fsm);
	dc = dp.getCanvas();
	f.add("Center", dp);


	Stabile s;
	Mobile m;
	if (inside) 
	    canvasSizeWorld = 2.25 * sradius;
	else 
	    canvasSizeWorld = 2.25 * (sradius + (2 * mradius));

	System.out.println("sradius=" + sradius);
	System.out.println("mradius=" + mradius);
	System.out.println("pradius=" + pradius);
	System.out.println("canvasSizeWorld=" + canvasSizeWorld);
	System.out.println("revs=" + revs);
	System.out.println("inside=" + inside);
	System.out.println("adaptive=" + adaptive);


	s = new CircleStabile(new PPoint(), sradius);
	m = new CircleMobile(mradius, pradius);
	double ia = 0.0;
	Design des = new Design(m, s, ia, (double)revs, canvasSizeWorld,
				siz, 1.25, 1.75, 660, inside);
	des.setAdaptive(adaptive);
	
	des.addObserver(new Observer() {
		public void update(Observable o, Object arg) {
		    if (arg instanceof Number)
			System.out.print(arg.toString() + ",");
		    else if (arg instanceof String) {
			Image i = ((Design)o).getImage(dc, 
						       Color.blue.darker(),
						       Color.white, false);
			dc.addImage(i, (Design)o);
		    }
		} });
	des.addObserver(dp);
	
	System.out.println("About to start thread..");
	Thread t1 = new Thread(des);
	f.pack();
	f.show();
	t1.start();
    }
     */

}

