package util;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

/**
 * ImageBlotter applies soft-edged blots to an image.
 * Given x,y coordinate as doubles in a given 
 * centered world coordinate system, it transforms 
 * them into image coordinate, and applies color to
 * nearby pixels according to their distance from the
 * virtual pixel.  The color application rule is very
 * simple: if the color being applied to the pixel 
 * is closer to the base color of the ImageBlot than
 * the color already in the pixel, then we apply the
 * color, otherwise we leave it alone.
 *
 * @see ImageBlot
 */

public class ImageBlotter  {
    private float     im[];
    private int       w;
    private int       h;
    
    private double wxe;		// world X extent +- 
    private double wye;		// world Y extent +-
				// (center always 0,0)
    
    private Graphics  g;
    private ImageBlot blot;

    private Color fg, bg;

    public ImageBlotter(double worldXExtent, double worldYExtent,
			int width, int height, 
			Color fgcolor, Color bgcolor,
			ImageBlot colorBlot)
    {
	fg = fgcolor;
	bg = bgcolor;
	w = width;
	h = height;
	im = null;
	wxe = worldXExtent;
	wye = worldYExtent;
	blot = colorBlot;
    }

    public void setBlot(ImageBlot b) { blot = b; }
    public ImageBlot getBlot() { return blot; }

    public Color getBackground() { return bg; }

    public synchronized void setColors(Color fgcolor, Color bgcolor) {
	if (fgcolor != null)
	    fg = fgcolor;
	if (bgcolor != null)
	    bg = bgcolor;
    }

    protected final void createArrayIfNeeded() {
	if (im == null) {
	    im = new float[w*h];
	    for(int c = 0; c < im.length; c++) {
		im[c] = 0.0f;
	    }
	}
    }

    public synchronized void release() {
	im = null;
	return;
    }

    public synchronized void reinitialize(Color fgcolor, Color bgcolor, 
			     ImageBlot colorBlot)
    {
	release();
	if (colorBlot != null) 
	    blot = colorBlot;
	if (fgcolor != null)
	    fg = fgcolor;
	if (bgcolor != null)
	    bg = bgcolor;
    }

    public MemoryImageSource memoryImageToSource(int [] imi, ColorModel cm) {
	if (cm == null) {
	    cm = ColorModel.getRGBdefault();
	}
	return new MemoryImageSource(w,h, cm, imi, 0, w);
    }

    public synchronized int [] addToMemoryImage(int [] imi) {
	if (im == null) return null;
	int bc, fcr, fcg, fcb, icr, icg, icb, dr, dg, db;
	int i, c;
	int len = im.length;
	if (imi == null) {
	    imi = new int[w*h];
	    bc = bg.getRed();
	    bc = (bc<<8) | bg.getGreen();
	    bc = (bc<<8) | bg.getBlue();
	    for(i = 0; i < len; i++) imi[i] = bc;
	}
	fcr = fg.getRed();
	fcg = fg.getGreen();
	fcb = fg.getBlue();
	for(i = 0; i < len; i++) {
	    c = imi[i];
	    icr = (c & 0x0ff0000) >> 16;
	    icg = (c & 0x000ff00) >> 8;
	    icb = c & 0x00000ff;
	    dr = fcr - icr;
	    icr = icr + (int)(im[i] * dr);
	    dg = fcg - icg;
	    icg = icg + (int)(im[i] * dg);
	    db = fcb - icb;
	    icb = icb + (int)(im[i] * db);
	    imi[i] = ((((icr & 0x0ff)<<8) | (icg & 0x0ff))<<8) | (icb & 0x0ff);
	}
	return imi;
    }

    public synchronized MemoryImageSource getProducer(ColorModel cm, boolean usealpha) {
	if (im == null) return null;
	if (cm == null) {
	    cm = ColorModel.getRGBdefault();
	}
	if (cm instanceof IndexColorModel) usealpha = false;
	int c;
	int imi[];
	imi = new int[im.length];
	int basered = bg.getRed();
	int addred = (fg.getRed() - basered);
	int basegreen = bg.getGreen();
	int addgreen = (fg.getGreen() - basegreen);
	int baseblue = bg.getBlue();
	int addblue = (fg.getBlue() - baseblue);
	int basepel;
	if (!usealpha) basepel = 255 << 24;
	else basepel = (fg.getRed() << 16)|(fg.getGreen() << 8)|fg.getBlue();
	for(c = 0; c < im.length; c++) {
	    if (usealpha) {
		imi[c] = ((((int)(255 * im[c])) & 0x00ff) <<24) | basepel;
	    }
	    else {
		imi[c] = basepel |
		    (((basered + (int)(addred * im[c]))&0x00ff)<<16) |
		    (((basegreen + (int)(addgreen * im[c]))&0x00ff)<<8) |
		    (((baseblue + (int)(addblue * im[c]))&0x00ff));
	    }
	}
	return new MemoryImageSource(w,h, cm, imi, 0, w);
    }

    public final double transformX(double x) {
	return w * ((x + wxe)/(2.0 * wxe));
    }
    public final double transformY(double y) {
	return h * ((y + wye)/(2.0 * wye));
    }
    public final double inverseTransformWidth(double x) {
	return (x / w) * 2.0 * wxe;
    }

    public void applyBlot(double x, double y) {
	createArrayIfNeeded();
	double pcx = transformX(x);
	double pcy = transformY(y);
	int minx = (int)(pcx - blot.bgrad);
	int maxx = (int)(pcx + blot.bgrad + 0.5);
	int miny = (int)(pcy - blot.bgrad);
	int maxy = (int)(pcy + blot.bgrad + 0.5);
	if (minx >= w || miny >= h || maxx < 0 || maxy < 0) return;
        if (minx < 0) minx = 0;
	if (miny < 0) miny = 0;
	if (maxx >= w) maxx = w - 1;
	if (maxy >= h) maxy = h - 1;
	int x1, y1;
	for(y1 = miny; y1 <= maxy; y1++) {
	    for(x1 = minx; x1 <= maxx; x1++) {
		im[w*y1 + x1] = 
		    blot.applyToPixel(im[w*y1 + x1], pcx, pcy,
				      (double)x1, (double)y1);
	    }
	}
	return;
    }

    /**
     * Main for testing...
     *
    static Image image1 = null;
    static Image image2 = null;
    static Color bgColor;
    public static void main(String [] args) {
	Color f = Color.yellow;
	Color b = Color.black;

	Frame fr = new Frame("ImageBlotter Test");
	bgColor = Color.green;
	Canvas c = new Canvas() {
		public void paint(Graphics g) {
		    if (image1 != null)
			g.drawImage(image1, 0, 0, bgColor, this);
		    if (image2 != null)
			g.drawImage(image2, 0, 0, this);
		}
	    };
	c.setSize(300,300);
	fr.add("Center", c);
	System.out.println("Creating ImageBlot..");
	ImageBlot blot = new ImageBlot(5, 12);
	System.out.println("Creating ImageBlotter..");
	ImageBlotter btest = new ImageBlotter(2.0, 2.0, 300,300,
					      f, b, blot);
	// apply some blots
	System.out.println("Applying some blots.");
	btest.applyBlot(0.0, 0.0);
	btest.applyBlot(-0.43, 0.25);
	btest.applyBlot(0.55, 0.55);
	btest.applyBlot(-0.55, -0.55);
	double x2, y2;
	for(x2 = 0.3, y2 = 0.4; x2 < 2.5; x2 += 0.002, y2 += 0.016) {
	    btest.applyBlot(x2,y2);
	}

	// get an image
	System.out.print("Creating the Image..");
	image1 = c.createImage(btest.getProducer(null, false));
	System.out.println("done.");
	
	// show the window
	fr.addWindowListener(new WindowAdapter() { 
		public void windowClosing(WindowEvent we) {
		    System.exit(0);
		}
	    });
	fr.pack();
	fr.show();

	// try a second test with alpha this time
	try { Thread.sleep(4000); } catch (Exception e) { }
	System.out.println("Creating new Blot.");
	blot = new ImageBlot(0.5,3.05);
	System.out.println("Reinitializing ImageBlotter.");
	btest.reinitialize(Color.white, Color.black, blot);
	System.out.println("Drawing Sin curve in blots.");
	for(x2 = -1.5; x2 < 1.75; x2 += 0.005) {
	    y2 = Math.sin(x2 * Math.PI * 2.0);
	    btest.applyBlot(x2, y2 - 0.91);
	}
	System.out.print("Creating second Image..");
	image2 = c.createImage(btest.getProducer(null, true));
	System.out.println("done.");
	c.repaint();
    }
    */
}
